//
// This is a part of software for the algorithm described in
//
// Kuzborskij, I., Carlucci, F. M., Caputo, B.
// "When Naive Bayes Nearest Neighbours Meet Convolutional Neural Networks".
// In: Computer Vision and Pattern Recognition (CVPR), 2016 IEEE Conference on.
//
// Contact the author: ilja.kuzborskij [at] idiap.ch
//                     http://idiap.ch/~ikuzbor
//
// ----------------------------------------------------------------------------------
//
// Copyright (c) 2015 Idiap Research Institute, http://www.idiap.ch/
// Written by Ilja Kuzborskij <ilja.kuzborskij@idiap.ch>
//
// This file is part of STOML3.
//
// STOML3 is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 3 as
// published by the Free Software Foundation.
//
// STOML3 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with STOML3. If not, see <http://www.gnu.org/licenses/>.
//

#include <assert.h>
#include <time.h>
#include <iostream>
#include <sstream>

using namespace std;

struct Timer {
    unsigned long _total;
    unsigned long _start;
    bool _paused;

    Timer()
        : _total(0), _start(0), _paused(true) {}

    inline void reset() {
        _total = 0;
    }

    inline void start() {
        assert(_paused);
        _paused = false;

        _start = clock();
    }

    inline void stop() {
        _total = clock() - _start;
        _start = 0;
    }

    inline void pause() {
        assert(!_paused);
        _paused = true;

        _total += (clock() - _start);
        _start = 0;
    }

    inline float get_ms() {
        return _total / (CLOCKS_PER_SEC/1000);
    }

    inline string get_string() {
        size_t msec = get_ms();
        stringstream ss;
        ss << msec/1000 << " seconds " << msec%1000 << " milliseconds.";
        return ss.str();
    }

    void save(ofstream &out) {
        out.write((char*) (&_total), sizeof(_total));
        out.write((char*) (&_start), sizeof(_start));
    }

    void load(ifstream &out) {
        out.read((char*) (&_total), sizeof(_total));
        out.read((char*) (&_start), sizeof(_start));
    }
};
